#include "langeditor.h"
#define MORETHANTHREELANGS false
LangEditor::LangEditor(QWidget *parent) : QWidget(parent)
{
    allLangs        = new QListWidget;
    nameEdit        = new QLineEdit;
    langkuerzelEdit = new QLineEdit;
    addBtn          = new QPushButton("Add");
    deleteBtn       = new QPushButton("delete");
    editConfBtn     = new QPushButton("Confirm");
    editAbortBtn    = new QPushButton("Abort");
    editBtn         = new QPushButton("Edit");

    QLabel* infoLabel = new QLabel("You need to save the database, if you want to use the newly created Languages");

    QGridLayout* layout = new QGridLayout;
    layout->addWidget(allLangs,0,0,7,1);
    layout->addWidget(addBtn,0,3);
    layout->addWidget(deleteBtn,1,3);
    layout->addWidget(editBtn,2,3);
    layout->addWidget(nameEdit,3,3,1,3);
    layout->addWidget(langkuerzelEdit,4,3,1,3);
    layout->addWidget(editConfBtn,5,3);
    layout->addWidget(editAbortBtn,5,4);
    layout->setRowStretch(6,1);
    layout->addWidget(infoLabel,7,0,1,6,Qt::AlignTop);
    this->setLayout(layout);
    curTile = Q_NULLPTR;
    editMode = false;

    nameEdit->setVisible(false);//wird nicht mehr gebraucht

    connect(addBtn      ,SIGNAL(clicked(bool)),this,SLOT(onAddBtn()));
    connect(deleteBtn   ,SIGNAL(clicked(bool)),this,SLOT(onDeleteBtn()));
    connect(editBtn     ,SIGNAL(clicked(bool)),this,SLOT(onEditBtn()));
    connect(editConfBtn ,SIGNAL(clicked(bool)),this,SLOT(onEditConfirmBtn()));
    connect(editAbortBtn,SIGNAL(clicked(bool)),this,SLOT(onEditAbortBtn()));

    this->setEnabled(false);
    ltype = Q_NULLPTR;
    onEditAbortBtn();
}
void LangEditor::newLangReference(QList<languageType>* q){
    ltype = q;
    resyncList();
    if(q == Q_NULLPTR)
        this->setEnabled(false);
    else
        this->setEnabled(true);
    if(ltype->count() >= 3 && !MORETHANTHREELANGS && !QApplication::arguments().contains("-NLL"))
        addBtn->setEnabled(false);
    else
        addBtn->setEnabled(true);
}
void LangEditor::resyncList(){
    if(ltype == Q_NULLPTR )return;
    allLangs->clear();
    if(ltype->count() >= 3 && !MORETHANTHREELANGS && !QApplication::arguments().contains("-NLL"))
        addBtn->setEnabled(false);
    else
        addBtn->setEnabled(true);
    for(int i = 0; i <  ltype->count();i++){
        QListWidgetItem* lwi = new QListWidgetItem();
        lwi->setText(QString(ltype->at(i).lang));
        if(ltype->at(i).job == DELETE_)
            lwi->setData(1,QColor(Qt::red));
        else if(ltype->at(i).job == ADD_)
            lwi->setData(1,QColor(Qt::cyan));
        else if(ltype->at(i).job == EDIT)
            lwi->setData(1,QColor(Qt::yellow));
        else
            lwi->setData(1,QColor(Qt::green));

        allLangs->addItem(lwi);
    }
}
void LangEditor::cleanAllData(){
    ltype=Q_NULLPTR;
    allLangs->clear();
    resyncList();
    this->setEnabled(false);
    onEditAbortBtn();
}
void LangEditor::onSelectionChanged(){
//    if(allLangs->selectedItems().count() == 0){
//        //platzhalter methode
//        return;
//    }
//    else{

//    }
}


void LangEditor::onAddBtn(){
    if(ltype==Q_NULLPTR)return;
    ltype->append(languageType(-1,"EXPL","EXAMPLE COUNTRY"));
    ((languageType*)&ltype->last())->job = ADD_;
    resyncList();
}
void LangEditor::onDeleteBtn(){
    int index = getCurrentSelectionIndex();
    if(ltype==Q_NULLPTR||index == -1)return;
    if(ltype->at(index).job == ADD_)
        ltype->removeAt(index);
    else if(ltype->at(index).job == DELETE_)
        ((languageType*)&ltype->at(index))->job = EDIT;
    else if(ltype->at(index).job != DELETE_)
        ((languageType*)&ltype->at(index))->job = DELETE_;

    resyncList();
}
int LangEditor::getCurrentSelectionIndex(){
    if(allLangs->selectedItems().count() == 0)return -1;
    return allLangs->row(allLangs->selectedItems().at(0));
}
void LangEditor::onEditBtn(){
    int ind = getCurrentSelectionIndex();
    if(ind == -1||ltype==Q_NULLPTR)return;
    //nameEdit->setText(ltype->at(ind).langName);
    langkuerzelEdit->setText(ltype->at(ind).lang);
    curTile = (languageType*)&ltype->at(ind);
    nameEdit->setEnabled(true);
    langkuerzelEdit->setEnabled(true);
    editMode = true;
    editAbortBtn->setEnabled(true);
    editConfBtn->setEnabled(true);
}
void LangEditor::onEditAbortBtn(){
    nameEdit->setText("");
    langkuerzelEdit->setText("");
    curTile = Q_NULLPTR;
    editMode = false;
    nameEdit->setEnabled(false);
    langkuerzelEdit->setEnabled(false);
    editAbortBtn->setEnabled(false);
    editConfBtn->setEnabled(false);
    resyncList();
}
void LangEditor::onEditConfirmBtn(){

    QString t = nameEdit->text();
    QString t2 = langkuerzelEdit->text();
    nameEdit->setText("");
    langkuerzelEdit->setText("");
    if(curTile == Q_NULLPTR)return;

    curTile->lang = t2;
    //curTile->langName = t;
    if(curTile->job!=ADD_)
        curTile->job = EDIT;
    curTile = Q_NULLPTR;
    editMode = false;
    nameEdit->setEnabled(false);
    langkuerzelEdit->setEnabled(false);
    editAbortBtn->setEnabled(false);
    editConfBtn->setEnabled(false);
    resyncList();
}
