#include "multilangimportwidget.h"
#include <QDebug>
MultiLangImportWidget::MultiLangImportWidget(QWidget *parent) : QWidget(parent)
{

    abortBtn       = new QPushButton("Abort");
    importBtn      = new QPushButton("Import");
    selectFileBtn  = new QPushButton("Select File");
    langCountSlider= new QSlider(Qt::Horizontal);
    this->setWindowTitle("Import a Multi Language *.CSV File");
    langCountSlider->setRange(1,5);//max is language Count
    fileD = new QFileDialog();
    fileD->setFileMode(QFileDialog::ExistingFile);
    fileD->setNameFilter(tr("CSV (*.csv *.CSV)"));
    wContainer  = new QWidget();
    sliderDescr = new QLabel("Languages to import: ");
    infoTxt     = new QLabel();
    filePathDescr  = new QLabel("File: ");
    scrollArea = new QScrollArea();
    scrollArea->setWidgetResizable(true);
    scrollArea->setWidget(wContainer);
    wContainer->setLayout(new QGridLayout());//IMPORTANT!
    QGridLayout* layout = new QGridLayout;

    layout->addWidget(sliderDescr,1,0);
    layout->addWidget(langCountSlider,1,1);
    layout->addWidget(scrollArea,2,0,1,3);
    layout->addWidget(filePathDescr,0,0);
    layout->addWidget(selectFileBtn,0,1);
    layout->addWidget(abortBtn,5,0);
    layout->addWidget(importBtn,5,2);
    layout->addWidget(infoTxt,8,0,1,3);
    this->setLayout(layout);

    connect(langCountSlider,SIGNAL(valueChanged(int)),this,SLOT(onSliderMoved()));
    connect(abortBtn,SIGNAL(clicked(bool)),this,SLOT(abort()));
    connect(selectFileBtn,SIGNAL(clicked(bool)),fileD,SLOT(show()));
    connect(importBtn,SIGNAL(clicked(bool)),this,SLOT(onImportBtn()));
    connect(fileD,SIGNAL(fileSelected(QString)),this,SLOT(onNewPath(QString)));
    onSliderMoved();
    QList<languageType> l;
//    l.append(languageType(1,"DE","GERMAN"));
//    l.append(languageType(2,"EN","English"));
    updateLang(l);
}
void MultiLangImportWidget::onSliderMoved(){
    sliderDescr->setText(QString(tr("Languages to import: ")).append(QString::number(langCountSlider->value())));
    syncComboBoxes();
    checkIfImportable();
}
void MultiLangImportWidget::updateLang(QList<languageType> ll){
    langList = ll;
    checkIfImportable();
    langCountSlider->setEnabled(langList.count()!=0);
    wContainer->setEnabled(langList.count()!=0);
    if(langList.count()!=0){
        langCountSlider->setRange(1,langList.count());
        infoTxt->setText("If you want to import more Languages, you need to add more into the Database");
    }
    else{
        langCountSlider->setRange(0,0);
        infoTxt->setText("Looks Like your Database does not have any Languages");
    }
    for(int i = 0; i < comboList.count();i++){
        updateComboBox((QComboBox*)comboList.at(i));
    }
}
void MultiLangImportWidget::openImportWidget(QList<languageType> ll){
    onNewPath("");
    updateLang(ll);
    this->show();
}
void MultiLangImportWidget::openImportWidget(){
    onNewPath("");
    this->show();this->setFocus();
}

void MultiLangImportWidget::abort()
{
    fileD->close();
    this->hide();
    emit onWindowClose();
}

void MultiLangImportWidget::onImportBtn()
{
    fileD->close();
    QList<int> langs;
    for(int i = 0; i < comboList.count();i++){
        int ind = comboList.at(i)->currentIndex();
        if(ind == -1){
            continue;
        }
        langs.append(langList.at(ind).getLangId());
    }
    emit import(filePath,langs);
    this->hide();
    emit onWindowClose();
}
void MultiLangImportWidget::onNewPath(QString q){
    fileD->close();
    filePath = q;
    checkIfImportable();
}
void MultiLangImportWidget::checkIfImportable(){
    bool isOkay = true;
    QList<int> s;
    for(int i = 0; i < comboList.count();i++){
        int ind = comboList.at(i)->currentIndex();

        if(ind == -1){
            isOkay = false;
            break;
        }

        s.append(langList.at(ind).getLangId());
    }
    int counter = 0;
    if(isOkay==true)
    for(int x = 0; x < s.count();x++){
        for(int i = 0; i < s.count();i++){
            if((int)s.at(x) == (int)s.at(i))
                counter++;
        }
        if(counter > 1){

            isOkay = false;
            break;
        }
        counter = 0;
    }
    importBtn->setEnabled((isOkay&&isValidPath() && langList.count()!=0));
}
bool MultiLangImportWidget::isValidPath(){
    return QFile::exists(filePath);
}
void MultiLangImportWidget::updateComboBox(QComboBox* cBox){
    if(cBox == Q_NULLPTR)return;
    cBox->blockSignals(true);
    cBox->clear();
    for(int i = 0; i < langList.count();i++){
        cBox->addItem(langList.at(i).lang);
    }
    cBox->blockSignals(false);
}
void MultiLangImportWidget::syncComboBoxes()
{
    int count = langCountSlider->value();

    QGridLayout* layout = (QGridLayout*)wContainer->layout();
    if(comboList.count() - count < 0){
        //add
        int val = -(comboList.count() - count);
        for(int i = 0; i < val;i++){
            QComboBox* box = new QComboBox();
            QLabel*    lab = new QLabel(tr("Lanugage ").append(QString::number(comboList.count()+i+1)));
            comboList.append(box);
            labelList.append(lab);
            layout->addWidget(lab,comboList.count()+i,0);
            layout->addWidget(box,comboList.count()+i,1);
            updateComboBox(box);

            connect(box,SIGNAL(currentIndexChanged(int)),this,SLOT(checkIfImportable()));

        }
    }
    else if( -(count-comboList.count()) > 0){
        int val = -(count-comboList.count());

        if(val < 0)return;
        for(int i = 0; i < val;i++){
            layout->removeWidget(comboList.last());
            layout->removeWidget(labelList.last());
            delete comboList.last();
            delete labelList.last();
            labelList.removeLast();
            comboList.removeLast();
        }
    }
}
void MultiLangImportWidget::updateText()
{
    onSliderMoved();
    this->setWindowTitle(tr("Multi Lang CSV Importer"));
}
void MultiLangImportWidget::closeEvent(QCloseEvent *event){
    emit this->onWindowClose();
    event->accept();
}
